#include "exportxml.h"




ExportXML::ExportXML(DataState *state, QFile *file, bool ViPERmode):state(state),file(file),ViPERmode(ViPERmode)
{

}

bool ExportXML::saveXML()
{
    if(!file->open(QIODevice::WriteOnly)){
        qDebug() <<"Could not open file location";
        // TODO: added user warning
        return false;
    }

    QXmlStreamWriter xmlWriter;

    xmlWriter.setDevice(file);

    xmlWriter.writeStartDocument();
    xmlWriter.setAutoFormatting(true);

    xmlWriter.writeStartElement("viper");
    xmlWriter.writeAttribute("xmlns","http://lamp.cfar.umd.edu/viper");
    xmlWriter.writeAttribute("xmlns:data","http://lamp.cfar.umd.edu/viperdata");

    xmlWriter.writeStartElement("config");

    xmlWriter.writeStartElement("descriptor");
    {
        QXmlStreamAttributes att;
         att.append("type","FILE");
         att.append("name","Information");
         xmlWriter.writeAttributes(att);

    }

    {//attributes in descriptor type="FILE" name="Information"
        xmlWriter.writeStartElement("attribute");
        QXmlStreamAttributes att;
        att.append("dynamic","false");
        att.append("type","dvalue");
        att.append("name","NUMFRAMES");
        xmlWriter.writeAttributes(att);
        xmlWriter.writeEndElement();
     }
    {
        xmlWriter.writeStartElement("attribute");
        QXmlStreamAttributes att;
        att.append("dynamic","false");
        att.append("type","dvalue");
        att.append("name","FPS");
        xmlWriter.writeAttributes(att);
        xmlWriter.writeEndElement();
    }
     {
        xmlWriter.writeStartElement("attribute");
        QXmlStreamAttributes att;
        att.append("dynamic","false");
        att.append("type","dvalue");
        att.append("name","H-FRAME-SIZE");
        xmlWriter.writeAttributes(att);
        xmlWriter.writeEndElement();
      }
    {
        xmlWriter.writeStartElement("attribute");
        QXmlStreamAttributes att;
        att.append("dynamic","false");
        att.append("type","dvalue");
        att.append("name","V-FRAME-SIZE");
        xmlWriter.writeAttributes(att);
        xmlWriter.writeEndElement();
    }
    xmlWriter.writeEndElement();

    foreach (TrackedObject *object,state->objectsList) {
          xmlWriter.writeStartElement("descriptor");
          {
               QXmlStreamAttributes att;
               att.append("type","OBJECT");
               att.append("name",object->name.replace(" ","_"));
               xmlWriter.writeAttributes(att);

          }
          {
               xmlWriter.writeStartElement("attribute");
               QXmlStreamAttributes att;
               if(object->getFrameSpans().size()==1 && object->getFrameSpans().first()->type==FrameSpan::Type::NotSet)
                    att.append("dynamic","false");
               else
               att.append("dynamic","true");
               att.append("type","bbox");
               att.append("name","BoundingBox");
               xmlWriter.writeAttributes(att);
               xmlWriter.writeEndElement();
          }
          xmlWriter.writeEndElement();
    }
    xmlWriter.writeEndElement();//end config
    xmlWriter.writeStartElement("data");

    xmlWriter.writeStartElement("sourcefile");
    xmlWriter.writeAttribute("filename",state->vid->videoFilePath.split("/").last()); // xmlWriter.writeAttribute("filename",framePath.split("/").last());

    xmlWriter.writeStartElement("file");
    {
         QXmlStreamAttributes att;
         att.append("id","0");
         att.append("name","Information");
         xmlWriter.writeAttributes(att);
    }
    {
        xmlWriter.writeStartElement("attribute");
        xmlWriter.writeAttribute("name","NUMFRAMES");
        xmlWriter.writeStartElement("data:dvalue");
        xmlWriter.writeAttribute("value",QString::number(state->vid->frameCount));
        xmlWriter.writeEndElement();
        xmlWriter.writeEndElement();
    }
    {
        xmlWriter.writeStartElement("attribute");
        xmlWriter.writeAttribute("name","FPS");
        xmlWriter.writeStartElement("data:dvalue");
        xmlWriter.writeAttribute("value",QString::number(state->vid->fps));
        xmlWriter.writeEndElement();
        xmlWriter.writeEndElement();
    }
    {
        xmlWriter.writeStartElement("attribute");
        xmlWriter.writeAttribute("name","H-FRAME-SIZE");
        xmlWriter.writeStartElement("data:dvalue");
        xmlWriter.writeAttribute("value",QString::number(state->vid->vidSize.height()));
        xmlWriter.writeEndElement();
        xmlWriter.writeEndElement();
    }
    {
        xmlWriter.writeStartElement("attribute");
        xmlWriter.writeAttribute("name","V-FRAME-SIZE");
        xmlWriter.writeStartElement("data:dvalue");
        xmlWriter.writeAttribute("value",QString::number(state->vid->vidSize.width()));
        xmlWriter.writeEndElement();
        xmlWriter.writeEndElement();
    }
    xmlWriter.writeEndElement();//end element

    foreach (TrackedObject* object ,state->objectsList) {
        QList<QXmlStreamAttributes> innerAtt;
        QList<QList<QXmlStreamAttributes>> innerInnerAttList;
        foreach(KeyFrame *keyFrame, object->keyFrames){
            if(!keyFrame->polygon.empty())
            {
              QXmlStreamAttributes att;
              QList<QXmlStreamAttributes> innerInnerAttLi;
              qreal x1,x2,y1,y2=0;
              keyFrame->polygon.boundingRect().getCoords(&x1,&y1,&x2,&y2);

              att.append("x",QString::number(round(x1)));
              att.append("y",QString::number(round(y1)));
              att.append("width",QString::number(round(x2-x1)));
              att.append("height",QString::number(round(y2-y1)));

              if(!ViPERmode){
                  const QList<QString> typeName({"NA","Manual","Manual","Automatic"});
                  const QList<QString> frameTypeName({"NA","Visible","Occluded"});

                  att.append("zorder",QString::number(keyFrame->zOrder));
                  att.append("annotationmode",typeName.at(keyFrame->type));
                  if(keyFrame->type==KeyFrame::Automatic)
                      att.append("objectmode", frameTypeName.at(object->frameSpans.at(keyFrame->frame)->type));
                  else
                      att.append("objectmode", frameTypeName.at(keyFrame->type));

//                  att.append("polyPts",polygon);

                  foreach(QPointF point,keyFrame->polygon.toList()) {
                      QXmlStreamAttributes innerInnerAtt;
                      innerInnerAtt.append("x",QString::number(round(point.x())));
                      innerInnerAtt.append("y",QString::number(round(point.y())));
                      innerInnerAttLi.push_back(innerInnerAtt);
                    }

              }

              att.append("framespan",QString::number(keyFrame->frame)+":"+QString::number(keyFrame->frame));
              innerAtt.push_back(att);
              innerInnerAttList.push_back(innerInnerAttLi);
              qDebug() << "frames"<< keyFrame->frame;
            }
        }

        if(!innerAtt.empty()){
        xmlWriter.writeStartElement("object");
        {
            QXmlStreamAttributes att;
            att.append("id",QString::number(object->id+1));
            QString framespan = innerAtt.first().value("framespan").toString().split(":").first() + ":" + innerAtt.last().value("framespan").toString().split(":").last();
            att.append("framespan",framespan);
            att.append("name",object->name.replace(" ","_"));
            xmlWriter.writeAttributes(att);
        }
       xmlWriter.writeStartElement("attribute");
       xmlWriter.writeAttribute("name",object->name.replace(" ","_"));
       foreach (QXmlStreamAttributes att, innerAtt){
           if(!ViPERmode){
            xmlWriter.writeStartElement("data:polygone");
            xmlWriter.writeAttributes(att);
            foreach (QXmlStreamAttributes attPoly, innerInnerAttList.takeFirst()) {
                xmlWriter.writeStartElement("data:point");
                xmlWriter.writeAttributes(attPoly);
                xmlWriter.writeEndElement();
            }
           }
           else{
                xmlWriter.writeStartElement("data:bbox");
                xmlWriter.writeAttributes(att);
           }
           xmlWriter.writeEndElement();
       }
       xmlWriter.writeEndElement();//end attribute
       xmlWriter.writeEndElement();//end object
        }
  }

  xmlWriter.writeEndElement();//end sourcefile
  xmlWriter.writeEndElement();//end data
  xmlWriter.writeEndElement();//end viper
  xmlWriter.writeEndDocument();

    return true;

}
