#include "keyframe.h"

#include <QDebug>
#include <QtCore>
#include "Geometry"

#include "trackedobject.h"
#include "../core/functions.h"
#include "../globalobjects.h"

KeyFrame::KeyFrame(TrackedObject* obj, int f, Type t) :
    QObject(obj),
    object(obj),
    frame(f),
    type(t)
{
    zOrder = obj->id;

}

void KeyFrame::reducePolygonVertices()
{
    QPolygonF tmpPoly = polygon;
    //qreal boundingDiag = Functions::distance(polygon.boundingRect().topLeft(), polygon.boundingRect().bottomRight()); //unused

    // iteratively remove points in polygon which satisfy some condition
    while(true) {
        qreal smallestIdx;
        qreal smallestRatio;

        for(int i=0; i<tmpPoly.length(); i++) {
            QPointF prev = tmpPoly[(i-1 + tmpPoly.length())%tmpPoly.length()];
            QPointF point = tmpPoly[i];
            QPointF next = tmpPoly[(i+1)%tmpPoly.length()];

            qreal prevNextDist = Functions::distance(prev, next);
            qreal lineSegDist = Functions::distToLineSegment(point, prev, next);

            // filtering condition is squared distance to line segment devided by length of line segment
            qreal ratio = lineSegDist*lineSegDist / (prevNextDist);

            if(i==0) {
                smallestIdx = 0;
                smallestRatio = ratio;
            } else if(ratio < smallestRatio) {
                smallestIdx = i;
                smallestRatio = ratio;
            }
        }
        // a value of 1.0 for the ratio threshold proved to be quite good
        // #magicNumbersRock
        qreal ratioThreshold = 1.0;
        if(smallestRatio > ratioThreshold) break;

        tmpPoly.remove(smallestIdx);
    }
    setNewPolygon(tmpPoly);
}

void KeyFrame::setNewPolygon(QPolygonF p)
{
    polygon = p;
    emit hasChanged(PolygonChange);
}

void KeyFrame::setType(KeyFrame::Type t)
{
    type = t;
    emit hasChanged(TypeChange);
}

void KeyFrame::transformPolygon(Eigen::Affine2d affineTransform)
{
    // apply transformation
    QPolygonF transformedPolygon = Functions::transformPolygonAffine(polygon,affineTransform);

    // filter intersection with image rect
    QRectF imageRect = g_mainWindow->frameViewer->pixmapItem->sceneBoundingRect();
    QPolygonF imagePoly(imageRect);
    QPolygonF intersectPoly = imagePoly.intersected(transformedPolygon);

    // This is due to an error with the QPolygonF::intersected method. The method will append
    // the first point of the polygon again at the end of it, if the first and last point were
    // not the same before. So just remove the last, if it is the same as the first.
    // This should lead to no complications as the first and last are always consecutive and
    // deletion of equal consecutive points should lead to no error.
    if(intersectPoly.first() == intersectPoly.last()) intersectPoly.removeLast();

    polygon = intersectPoly;
    emit hasChanged(PolygonChange);
}

