#ifndef KEYFRAME_H
#define KEYFRAME_H

#include <QObject>
#include <QPolygonF>
#include <QDataStream>
#include "Dense"

class TrackedObject;

class KeyFrame : public QObject
{
    Q_OBJECT
public:
    enum Type {
        NotSet,
        SetVisible,
        SetOccluded,
        Automatic
    };

    enum ChangeFlags {
        PolygonChange = 1<<0,
        TypeChange = 1<<1,
        ZOrderChange = 1<<2
    };

    explicit KeyFrame(TrackedObject* obj, int f, Type t);

    //define << operator
    friend QDataStream& operator<<(QDataStream& out, const KeyFrame &outPut){
        out<<outPut.frame;
        out<<int(outPut.type);
        out<<outPut.polygon;
        out<<outPut.zOrder;
        return out;
    }
    //define >> operator
    friend QDataStream& operator>>(QDataStream& in, KeyFrame &inPut){
        in>>inPut.frame;
        int tmp=0;
        in>>tmp;
        inPut.type=Type(tmp);
        in>>inPut.polygon;
        in>>inPut.zOrder;
        return in;
    }

    void reducePolygonVertices();

    TrackedObject* object;
    int frame;
    Type type;
    QPolygonF polygon;

    int zOrder;



signals:
    void hasChanged(int f);

public slots:
    void setNewPolygon(QPolygonF p);
    void setType(Type t);
    void transformPolygon(Eigen::Affine2d affineTransform);


};

#endif // KEYFRAME_H
