#ifndef TRACKEDOBJECT_H
#define TRACKEDOBJECT_H

#include <QObject>
#include <QList>
#include <QDataStream>
#include <QPolygonF>
#include <QDebug>
#include <QColor>

#include "keyframe.h"
#include "framespan.h"

class TrackedObject : public QObject
{
    Q_OBJECT
public:
    static int newID;

    enum ChangeTypes {
        NameChange = 1<<0,
        FrameSpanChange = 1<<1,
        ZOrderChange = 1<<2
    };

    explicit TrackedObject(QObject *parent = 0);

    //define << operator
    friend QDataStream& operator<<(QDataStream& out, const TrackedObject &outPut){
        out<<outPut.name;
        out<<outPut.id;
        for (int i = 0; i < outPut.keyFrames.size(); i++){
            out<<*(outPut.keyFrames.at(i));
        }
        for (int i = 0; i < outPut.frameSpans.size()-1; i++){
            out<<*(outPut.frameSpans.at(i));
            qDebug()<<outPut.frameSpans.at(i)->nextFrame;
            i=outPut.frameSpans.at(i)->nextFrame-1;
        }
        return out;
    }
    //define >> operator
    friend QDataStream& operator>>(QDataStream& in, TrackedObject &inPut){
        in>>inPut.name;
        in>>inPut.id;
        for (int i = 0; i < inPut.keyFrames.size(); i++){
            in>>*(inPut.keyFrames.at(i));
        }
        for (int i = 0; i < inPut.frameSpans.size(); i++){
            if(i==0){
                in>>*(inPut.frameSpans.at(i));
            }
            else{
                FrameSpan* span = new FrameSpan(&inPut, FrameSpan::NotSet, i, i+1);
                in>>*span;
                for(int frame = span->prevFrame; frame <=span->nextFrame; frame++) {
                    inPut.frameSpans.replace(frame,span);
                }
            }

            i=inPut.frameSpans.at(i)->nextFrame;
            if(i>=inPut.frameSpans.size()-1)
                break;
        }
        return in;
    }

    QString name;
    int id;
    QList<KeyFrame*> keyFrames;
    QList<FrameSpan*> frameSpans;

    void changeName(QString txt);
    bool isCurrent();
    KeyFrame* getCurrentKeyFrame();
    QList<FrameSpan *> getFrameSpans();

    struct TrackedObjectLess {
        int compFrame;
        TrackedObjectLess(int f) {
            compFrame = f;
        }
        bool operator() (TrackedObject *a, TrackedObject *b) {
            return a->keyFrames[compFrame]->zOrder < b->keyFrames[compFrame]->zOrder;
        }
    };
    static TrackedObjectLess combareByZOrderOfFrame(int frame);

    int getZOrderRank(int frame);

    QColor color;

signals:
    void hasChanged(int f);

public slots:
    void keyFrameChanged(int f);


};

#endif // TRACKEDOBJECT_H
