#include "framespangraphicsitem.h"


#include <QDebug>
#include <QMenu>
#include <QAction>
#include <QActionGroup>

#include "../Models/trackedobject.h"

FrameSpanGraphicsItem::FrameSpanGraphicsItem(QGraphicsScene* s, QPointF p1, QPointF p2, FrameSpan *fs) :
    span(fs)
{


    QPainterPath p;
    p.moveTo(p1);

    if(p1.y() == p2.y()) {
        p.lineTo(p2);
    } else {
        qreal x_half = (p2.x() + p1.x()) / 2.0;
        p.cubicTo(QPointF(x_half, p1.y()), QPointF(x_half, p2.y()), p2);
    }

    setPath(p);
    loadStyle();

    s->addItem(this);

    connect(span, SIGNAL(hasChanged(int)), this, SLOT(reload()));
}

void FrameSpanGraphicsItem::reload()
{
    loadStyle();
}

void FrameSpanGraphicsItem::loadStyle()
{
    QColor c = this->span->object->color;
    if(span->type == FrameSpan::NotSet) {
        setPen(QPen(c, 1, Qt::DotLine));
    }
    if(span->type == FrameSpan::Visible) {
        setPen(QPen(c, 3, Qt::SolidLine));
    }
    if(span->type == FrameSpan::Occluded) {
        setPen(QPen(c, 3, Qt::DashLine));
    }
}

void FrameSpanGraphicsItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
    QMenu menu(0);
    QActionGroup typegroup(this);

    QAction *typeVisibleAction = menu.addAction("Visible");
    typeVisibleAction->setCheckable(true);
    typeVisibleAction->setActionGroup(&typegroup);
    typeVisibleAction->setChecked(span->type == FrameSpan::Visible);

    QAction *typeOccludedAction = menu.addAction("Occluded");
    typeOccludedAction->setCheckable(true);
    typeOccludedAction->setActionGroup(&typegroup);
    typeOccludedAction->setChecked(span->type == FrameSpan::Occluded);

    QAction *typeHiddenAction = menu.addAction("Hidden");
    typeHiddenAction->setCheckable(true);
    typeHiddenAction->setActionGroup(&typegroup);
    typeHiddenAction->setChecked(span->type == FrameSpan::NotSet);


    QAction *selectedAction = menu.exec(event->screenPos());
    if ( selectedAction != 0 ) {
        if(selectedAction == typeVisibleAction && span->type != FrameSpan::Visible) {
            span->setType(FrameSpan::Visible);
        }
        if(selectedAction == typeOccludedAction && span->type != FrameSpan::Occluded) {
            span->setType(FrameSpan::Occluded);
        }
        if(selectedAction == typeHiddenAction && span->type != FrameSpan::NotSet) {
            span->setType(FrameSpan::NotSet);
        }
    }
}

