#include "trackedobjectlabel.h"

#include <QHBoxLayout>

#include "../globalobjects.h"

TrackedObjectLabel::TrackedObjectLabel(int id) :
    id(id),
    clicked(false)
{
    setFixedHeight(40);

    updateStyle();
    connect(g_state, SIGNAL(hasChanged(int)), this, SLOT(filterStateChange(int)));

    QHBoxLayout *l = new QHBoxLayout();
    this->setLayout(l);
    l->setContentsMargins(0,0,0,0);
    l->setSpacing(0);

    labelStack = new QStackedWidget();

    label = new QLabel(g_state->getObjectById(id)->name);
    labelStack->addWidget(label);

    labelEdit = new QLineEdit(g_state->getObjectById(id)->name);
    connect(labelEdit, SIGNAL(editingFinished()), this, SLOT(changeName()));
    labelStack->addWidget(labelEdit);

    l->addWidget(labelStack);


    buttonEdit = new QToolButton();
    buttonEdit->setText("Edit");
    buttonEdit->setIcon(QIcon(":icons/edit.ico"));
    buttonEdit->setIconSize(QSize(29,29));
    connect(buttonEdit, SIGNAL(clicked(bool)), this, SLOT(toggleEdit()));
    l->addWidget(buttonEdit);

    buttonDelete = new QToolButton();
    buttonDelete->setText("Del");
    buttonDelete->setIcon(QIcon(":icons/del.ico"));
    buttonDelete->setIconSize(QSize(29,29));

    connect(buttonDelete, SIGNAL(clicked(bool)), this, SLOT(deleteObj()));
    l->addWidget(buttonDelete);

}

void TrackedObjectLabel::changeName()
{
    if(labelEdit->text() != label->text()) {
        g_state->getObjectById(id)->changeName(labelEdit->text());
    }
}

void TrackedObjectLabel::deleteObj()
{
    g_state->deleteObject(id);
    if(!g_state->hasObjects()){
        g_mainWindow->reload();
    }
    g_mainWindow->timeLine->reload();
}

void TrackedObjectLabel::toggleEdit()
{
    if(labelStack->currentWidget() == label) {
        labelStack->setCurrentWidget(labelEdit);
        labelEdit->setFocus();
        labelEdit->selectAll();
    } else if(labelStack->currentWidget() == labelEdit) {
        labelEdit->clearFocus();
        labelStack->setCurrentWidget(label);
    }
}

void TrackedObjectLabel::updateStyle()
{
    QPalette pal;
    if(g_state->getObjectById(id)->isCurrent()) {
        pal.setColor(QPalette::Background, Qt::red);
    } else {
        pal.setColor(QPalette::Background, Qt::white);
    }
    setAutoFillBackground(true);
    setPalette(pal);
}

void TrackedObjectLabel::filterStateChange(int f)
{
    if(f & DataState::CurrentObjectChange) updateStyle();
}

void TrackedObjectLabel::mousePressEvent(QMouseEvent *event)
{
    if(id != g_state->currentObjectID) {
        g_state->setCurrentObject(id);
    }
    QFrame::mousePressEvent(event);
}

void TrackedObjectLabel::mouseDoubleClickEvent(QMouseEvent *event)
{
    toggleEdit();
}

